/*
 *  arpatk
 *  src/arpatk.h
 *
 *  Copyright (C) 2011
 *
 *  Author: Vilmain Nicolas
 *  Contact: null.sim@gmail.com
 *
 *  This file is part of arpatk.
 *
 *  arpatk is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  arpatk is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with arpatk.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __ARPATK_H__
#define __ARPATK_H__

#include  "ostype.h"

#if !defined(__BSD) && !defined(__Linux)
# error "set macros __Linux or __BSD"
#endif /* not __Linux and not __BSD */

#include  <time.h>
#include  <fcntl.h>
#include  <errno.h>
#include  <stdio.h>
#include  <stdlib.h>
#include  <stdint.h>
#include  <string.h>
#include  <unistd.h>
#include  <stdarg.h>
#include  <signal.h>
#include  <sys/time.h>
#include  <sys/ioctl.h>
#include  <netinet/in.h>
#include  <arpa/inet.h>
#include  <sys/socket.h>
#ifdef __Linux
# include  <linux/if.h>
# include  <linux/if_ether.h>
# include  <linux/if_packet.h>
#elif defined (__BSD)
# include  <net/if.h>
# include  <net/if_dl.h>
# include  <net/bpf.h>
# include  <net/ethernet.h>
#endif /* not __Linux */
#include  <ifaddrs.h>

#define  ARP_SIZE        sizeof (struct arp_packet_s)
#define  NAME_LEN        16
#define  STR_IP_LEN      17
#define  MAC_SIZE        6
#define  IP4_SIZE        4
#define  PACKET_SIZE     65535

#define  ARP_OPCODE_REQUEST 1
#define  ARP_OPCODE_REPLY   2

/* error because strict aliasing !
   you need compiling with flags -fno-strict-aliasing for use
   this macro ... but replace by memcpy ... it's more easy
#define COPY_MAC(dst, src)					\
  (*(uint32_t *) dst)       =   (*(uint32_t *) src);		\
  (*(uint16_t *) (dst + 4)) =   (*(uint16_t *) (src + 4))
*/

#define  NULL_MAC              (uint8_t *) "\x00\x00\x00\x00\x00\x00"
#define  BROADCAST_ADDR        (uint8_t *) "\xff\xff\xff\xff\xff\xff"
#define  COPY_MAC(dst, src)    memcpy (dst, src, MAC_SIZE)
#define  CMP_MAC(mac1, mac2)   memcmp (mac1, mac2, MAC_SIZE)
#define  PRINT_MAC_ADDRS(mac)				\
  mac[0] & 0xff, mac[1] & 0xff, mac[2] & 0xff,		\
  mac[3] & 0xff, mac[4] & 0xff, mac[5] & 0xff

#define  COPY_IP(dst, src)     memcpy (dst, src, IP4_SIZE)
#define  CMP_IP(ip1, ip2)      memcmp (ip1, ip2, IP4_SIZE)
#define  CHECK_MASK(mask, ip)  memcmp (mask, ip, 3)

/* protocol list */
enum
  {
#ifdef __Linux
    ETH_IP = ETH_P_IP,
    ETH_ARP = ETH_P_ARP,
#elif defined(__BSD)
    ETH_IP = ETHERTYPE_IP,
    ETH_ARP = ETHERTYPE_ARP,
#endif /* not __Linux */
  };

struct netconf_s
  {
    int nc_index;
    int nc_mtu;
    char nc_name[NAME_LEN];
    uint8_t nc_mac[MAC_SIZE];
    struct in_addr nc_ipv4;
  };

struct target_s
  {
    uint8_t mac_spoof[MAC_SIZE];
    uint8_t mac_target[MAC_SIZE];
    struct in_addr ip_spoof;
    struct in_addr ip_target;
    char str_ip_spf[STR_IP_LEN];
    struct target_s *next;
  };

struct ethsock_s
  {
    int fdsock;
#ifdef __Linux
    struct sockaddr_ll ll;
# define   SOCK_LL_SIZE   sizeof (struct sockaddr_ll)
#endif /* __Linux */
  };

struct arp_packet_s
  {
    uint8_t eth_dst[MAC_SIZE];
    uint8_t eth_src[MAC_SIZE];
    uint16_t eth_protocol;
    uint16_t arp_hrd;
    uint16_t arp_protocol;
    uint8_t arp_hln;
    uint8_t arp_pln;
    uint16_t arp_opcode;
    uint8_t arp_mac_src[MAC_SIZE];
    uint8_t arp_ip_src[IP4_SIZE];
    uint8_t arp_mac_dst[MAC_SIZE];
    uint8_t arp_ip_dst[IP4_SIZE];
  } __attribute__((packed));
#define ARP  struct arp_packet_s

int     get_netconf (struct netconf_s *nc, const char *name);
void *  xcalloc (size_t size);
int     xstrtol_positiv_value (const char *str, int *dec);
int     get_uchar_mac (const char *str, uint8_t *mac);
int     get_ethsock (struct ethsock_s *s, int index, const char *name);
int     ethsend (struct ethsock_s *s, void *packet, size_t size);
int     ethrecv(int fdsock, void *packet);

char *program_name;

#endif /* not __ARPATK_H__ */
