/*
 *  arpatk
 *  src/utils.c
 *
 *  Copyright (C) 2011
 *
 *  Author: Vilmain Nicolas
 *  Contact: null.sim@gmail.com
 *
 *  This file is part of arpatk.
 *
 *  arpatk is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  arpatk is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with arpatk.  If not, see <http://www.gnu.org/licenses/>.
 */

#include  "arpatk.h"

static void  out_memory (const char *err);

static void
out_memory (const char *err)
{
  fprintf (stderr, "%s:%s: memory exhausted\n",
	   program_name, err);
  exit (EXIT_FAILURE);
}

void *
xcalloc (size_t size)
{
  void *ptr = NULL;

  ptr = malloc (size);
  if (!ptr)
    out_memory ("malloc");
  memset (ptr, 0, size);
  return ptr;
}

int
xstrtol_positiv_value (const char *str, int *dec)
{
  char *err = NULL;

  *dec = strtol (str, &err, 0);
  if (*err != '\0')
    {
      fprintf (stderr, "%s: %s is not integer value\n",
	       program_name, str);
      return -1;
    }
  if (*dec < 1)
    {
      fprintf (stderr, "%s: %s is not positiv value\n",
	       program_name, str);
      return -1;
    }
  return 0;
}

int
get_uchar_mac (const char *str, uint8_t *mac)
{
  uint8_t *p_mac = NULL;
  char buf[3];
  int i;

  if (strlen (str) != 17
      || str[2] != ':' || str[5] != ':'
      || str[8] != ':' || str[11] != ':'
      || str[14] != ':')
    return -1;
  i = 0;
  p_mac = mac;
  buf[2] = '\0';
  do
    {
      buf[0] = str[i];
      buf[1] = str[i + 1];
      *p_mac++ = (uint8_t) strtol (buf, NULL, 16);
    }
  while ((i += 3) < 17);
  return 0;
}
